import type { AnyRequest } from './request-types';
export type FuncOrConfigure<T extends Function, TConfig> = T & {
    withConfig(config: TConfig): {
        execute: T;
    };
};
export type ProxyFor<TFac, TConfig> = ProxyForInternal<TFac, keyof TFac, TConfig>;
export type ProxyForInternal<TFac, TKey extends keyof TFac, TConfig> = {
    [key in TKey]: TFac[key] extends (...args: infer TArgs) => AnyRequest<infer TResponse> ? FuncOrConfigure<(...args: TArgs) => Promise<TResponse>, TConfig> : TFac[key];
};
export type ProxyForWithRequiredConfig<TFac, TConfig> = ProxyForWithRequiredConfigInternal<TFac, keyof TFac, TConfig>;
export type ProxyForWithRequiredConfigInternal<TFac, TKey extends keyof TFac, TConfig> = {
    [key in TKey]: TFac[key] extends (...args: infer TArgs) => AnyRequest<infer TResponse> ? (arg0: TConfig, ...args: TArgs) => Promise<TResponse> : TFac[key];
};
export type MakeRequest<TConfig> = <TResponse>(request: AnyRequest<TResponse>, config: TConfig | undefined) => Promise<TResponse>;
export type NotDefaultedConfig<TConfig, TDefaultedConfig extends Partial<TConfig>> = Partial<TDefaultedConfig> & Omit<TConfig, keyof TDefaultedConfig> extends infer O ? {
    [key in keyof O]: O[key];
} : never;
export declare class ApiProxyFactory<TConfig = never> {
    private makeRequest;
    constructor(makeRequest: MakeRequest<TConfig>);
    createProxy<TRequestFactory extends object, TConfigDefaults extends Partial<TConfig>>(requestFactory: TRequestFactory, defaultConfig?: TConfigDefaults): ProxyFor<TRequestFactory, NotDefaultedConfig<TConfig, TConfigDefaults>>;
    createProxyWithRequiredConfig<TRequestFactory extends object, TConfigDefaults extends Partial<TConfig>>(requestFactory: TRequestFactory, defaultConfig?: TConfig): ProxyForWithRequiredConfig<TRequestFactory, NotDefaultedConfig<TConfig, TConfigDefaults>>;
    private createProxyInternal;
}
