class ApiProxyFactory {
    constructor(makeRequest) {
        this.makeRequest = makeRequest;
    }
    createProxy(requestFactory, defaultConfig) {
        return this.createProxyInternal(requestFactory, false, defaultConfig);
    }
    createProxyWithRequiredConfig(requestFactory, defaultConfig) {
        return this.createProxyInternal(requestFactory, true, defaultConfig);
    }
    createProxyInternal(requestFactory, firstArgIsConfig, defaultConfig) {
        const factory = this;
        const proxyCache = {};
        return new Proxy(requestFactory, {
            get(target, property, receiver) {
                if (typeof property === 'string' && typeof target[property] === 'function' && property !== 'prototype') {
                    if (proxyCache[property])
                        return proxyCache[property];
                    async function makeRequestIntl(config, args) {
                        const requestFactory = Reflect.get(target, property, receiver);
                        const request = requestFactory(...args);
                        try {
                            return factory.makeRequest(request, defaultConfig ? { ...defaultConfig, ...config } : config);
                        }
                        catch (error) {
                            console.error('Api error', error);
                            throw error;
                        }
                    }
                    function makeRequest(...args) {
                        const [config, otherArgs] = splitConfig(args, firstArgIsConfig);
                        return makeRequestIntl(config, otherArgs);
                    }
                    makeRequest.withConfig = firstArgIsConfig
                        ? undefined
                        : (config) => {
                            return {
                                execute: (...args) => {
                                    return makeRequestIntl(config, args);
                                },
                            };
                        };
                    return (proxyCache[property] = makeRequest);
                }
                return Reflect.get(target, property, receiver);
            },
        });
    }
}
function splitConfig(args, firstArgIsConfig) {
    if (!firstArgIsConfig)
        return [undefined, args];
    return [args[0], args.slice(1)];
}

export { ApiProxyFactory };
//# sourceMappingURL=api-proxy-factory.mjs.map
